@@ -620,7 +620,6 @@ typedef struct r_anal_t {
 	int bits;
 	int lineswidth; // wtf
 	int big_endian;
-	int split; // used only from core
 	int sleep; // sleep some usecs before analyzing more (avoid 100% cpu usages)
 	RAnalCPPABI cpp_abi;
 	void *user;
@@ -1287,6 +1286,7 @@ R_API const char *r_anal_stackop_tostring (int s);
 R_API RAnalOp *r_anal_op_new(void);
 R_API void r_anal_op_free(void *op);
 R_API bool r_anal_op_fini(RAnalOp *op);
+R_API RAnalVar *get_link_var(RAnal *anal, ut64 faddr, RAnalVar *var);
 R_API bool r_anal_op_is_eob (RAnalOp *op);
 R_API RList *r_anal_op_list_new(void);
 R_API int r_anal_op(RAnal *anal, RAnalOp *op, ut64 addr,
