@@ -19,8 +19,7 @@ typedef enum {
 	//RNCXOR='^', RNCOR='|', RNCAND='&',
 	RNCNEG='~', RNCAND='&', RNCORR='|', RNCXOR='^',
 	RNCPRINT=';', RNCASSIGN='=', RNCLEFTP='(', RNCRIGHTP=')',
-	RNCSHL='<', RNCSHR = '>'
-
+	RNCSHL='<', RNCSHR = '>', RNCROL = '#', RNCROR = '$'
 } RNumCalcToken;
 
 typedef struct r_num_calc_t {
@@ -33,6 +32,7 @@ typedef struct r_num_calc_t {
 	int calc_i;
 	const char *calc_buf;
 	int calc_len;
+	bool under_calc;
 } RNumCalc;
 
 typedef struct r_num_t {
@@ -49,6 +49,9 @@ typedef struct r_num_t {
 typedef ut64 (*RNumCallback)(struct r_num_t *self, const char *str, int *ok);
 typedef const char *(*RNumCallback2)(struct r_num_t *self, ut64, int *ok);
 
+void r_srand(int seed);
+int r_rand(int mod);
+
 R_API RNum *r_num_new(RNumCallback cb, RNumCallback2 cb2, void *ptr);
 R_API void r_num_free(RNum *num);
 R_API char *r_num_units(char *buf, ut64 num);
@@ -72,7 +75,11 @@ R_API void r_num_irand(void);
 R_API ut16 r_num_ntohs(ut16 foo);
 R_API ut64 r_get_input_num_value(RNum *num, const char *input_value);
 R_API int r_is_valid_input_num_value(RNum *num, const char *input_value);
-R_API int r_num_between (RNum *num, const char *input_value);
+R_API int r_num_between(RNum *num, const char *input_value);
+R_API bool r_num_is_op(const char c);
+R_API int r_num_str_len(const char *str);
+R_API int r_num_str_split(char *str);
+R_API RList *r_num_str_split_list(char *str);
 
 #ifdef __cplusplus
 }
